\ Forth-Words for Nucleo F303K8 for PAS-control

compiletoflash

decimal

\ Cornerstone for 2 kByte Flash pages

: cornerstone ( Name ) ( -- )
  <builds begin here $7FF and while 0 h, repeat
  does>   begin dup  $7FF and while 2+   repeat 
          eraseflashfrom
;

cornerstone Rewind-to-Core


\ Utilities

$48000000 constant PORTA
$48000400 constant PORTB
$40021000 constant RCC

PORTA  $0 + constant GPIOA_MODER      	\ address for mode register
PORTA $14 + constant GPIOA_ODR        	\ address for output data

PORTB  $0 + constant GPIOB_MODER      	\ address for mode register
PORTB $14 + constant GPIOB_ODR        	\ address for output data

\ bit masks
1 0 lshift constant BIT0	
1 1 lshift constant BIT1
1 2 lshift constant BIT2
1 3 lshift constant BIT3
1 4 lshift constant BIT4
1 5 lshift constant BIT5
1 6 lshift constant BIT6
1 7 lshift constant BIT7
1 8 lshift constant BIT8
1 9 lshift constant BIT9
1 10 lshift constant BIT10
1 11 lshift constant BIT11
1 12 lshift constant BIT12
1 13 lshift constant BIT13
1 14 lshift constant BIT14
1 15 lshift constant BIT15
1 16 lshift constant BIT16
1 17 lshift constant BIT17
1 18 lshift constant BIT18
1 19 lshift constant BIT19
1 20 lshift constant BIT20
1 21 lshift constant BIT21
1 22 lshift constant BIT22
1 23 lshift constant BIT23
1 24 lshift constant BIT24
1 25 lshift constant BIT25
1 26 lshift constant BIT26
1 27 lshift constant BIT27
1 28 lshift constant BIT28
1 29 lshift constant BIT29
1 30 lshift constant BIT30
1 31 lshift constant BIT31
		
: 1ms 1850 0 do loop ;   \ this is only approximate!
: ms 0 do 1ms loop ;     



\ SPI1 setup

\ SPI1 with SCK on PB3, MISO on PB4, MOSI on PB5
\ same SPI pin-out as Arduino Nano

\ register definitions

PORTB $20 + constant GPIOB_AFRL
RCC $18 + constant RCC_APB2ENR
$40013000 constant SPI1
SPI1 $00 + constant SPI1_CR1
SPI1 $04 + constant SPI1_CR2
SPI1 $08 + constant SPI1_SR
SPI1 $0C + constant SPI1_DR

\ configuring the MOSI, SCK and MISO pins

: SPI1PINSETUP				
  BIT6  GPIOB_MODER bic!		\ set to alternate function, PB3
  BIT7  GPIOB_MODER bis!
  BIT8  GPIOB_MODER bic!		\ PB4
  BIT9  GPIOB_MODER bis!
  BIT10 GPIOB_MODER bic!		\ PB5
  BIT11 GPIOB_MODER bis!
  BIT12 GPIOB_AFRL  bis!		\ set the mode, AF5, %0101, set to SPI, PB3
  BIT13 GPIOB_AFRL  bic!
  BIT14 GPIOB_AFRL  bis!			
  BIT15 GPIOB_AFRL  bic!
  BIT16 GPIOB_AFRL  bis!		\ AF5, %0101, set to SPI, PB4
  BIT17 GPIOB_AFRL  bic!
  BIT18 GPIOB_AFRL  bis!			
  BIT19 GPIOB_AFRL  bic!
  BIT20 GPIOB_AFRL  bis!		\ AF5, %0101, set to SPI, PB5
  BIT21 GPIOB_AFRL  bic!
  BIT22 GPIOB_AFRL  bis!			
  BIT23 GPIOB_AFRL  bic!
;

: SPI1SETUP ( mode -- ) 
  BIT12 RCC_APB2ENR bis!    		\ enable clock for SPI1
  SPI1PINSETUP
  BIT6 SPI1_CR1 bic!   			\ SPI disable
  BIT9 SPI1_CR1 bis!   			\ SSM for software SS
  BIT8 SPI1_CR1 bis!   			\ SSI otherwise get MODF error, p. 974
  case
    0 of BIT0 SPI1_CR1 bic! BIT1 SPI1_CR1 bic! endof  \ standard modes, not the TI modes
    1 of BIT0 SPI1_CR1 bis! BIT1 SPI1_CR1 bic! endof  \ BIT0 = CPHA
    2 of BIT0 SPI1_CR1 bic! BIT1 SPI1_CR1 bis! endof  \ BIT1 = CPOL
    3 of BIT0 SPI1_CR1 bis! BIT1 SPI1_CR1 bis! endof  \ 
  endcase 
  BIT2  SPI1_CR1 bis!   		\ Master selection
  BIT3  SPI1_CR1 bis!   		\ fPCLK/64 should give 125 kHz
  BIT4  SPI1_CR1 bic!   		\ fPCLK/64
  BIT5  SPI1_CR1 bis!   		\ fPCLK/64
  BIT12 SPI1_CR2 bis!   		\ 8 bit receive event
  BIT6  SPI1_CR1 bis!   		\ SPI enable
;

: SPI ( u1 -- u2 )
  begin  BIT1 SPI1_SR bit@ until SPI1_DR c!   
  begin  BIT0 SPI1_SR bit@ until SPI1_DR c@
;

: C!SPI  ( u -- )  spi drop ;
: C@SPI  ( -- u )  $FF spi ;


\ Using PA11 for default CS (corresponds to the Arduino Nano default)

: CSSETUP-PA11 		                \ set PA11 as output, is not default on reset!
  BIT22 GPIOA_MODER bis!		\ there is no easy way to clear and set at same time
  BIT23 GPIOA_MODER bic! 		\ so adjust bits individually
;   

: CSON-PA11  BIT11 GPIOA_ODR bic! ;     \ turn PA11 lo to turn on chip select (active lo)
: CSOFF-PA11  BIT11 GPIOA_ODR bis! ;    \ turn PA11 hi to turn off chip select


\ Using PA8 as alternate CS (next to the PA11 pin)

: CSSETUP-PA8 		                
  BIT16 GPIOA_MODER bis!		
  BIT17 GPIOA_MODER bic! 		
;   

: CSON-PA8  BIT8 GPIOA_ODR bic! ; 
: CSOFF-PA8  BIT8 GPIOA_ODR bis! ; 

\ example of SPI usage with the MAX522 DAC

\ : MAX522SETUP ( -- )
\   0 SPI1SETUP            \ mode 0 
\   CSSETUP-PA11               
\ ;

\ : MAX522C! ( u -- )
\   CSON-PA11                  
\   $23 C!SPI C!SPI       \ $23 is the register set-up for the MAX522, $21 also works
\   CSOFF-PA11                 
\ ;



\ AD9833

\ first the calculation with the formula

: FREQREGCALC   ( ud -- u )          \  put decimal point at end of number
    268435456. udm* 2drop            \  2^28, 2drop removes overflow beyond 64 bits
    25000000. ud/mod 2swap 2drop     \  for a 25 MHz crystal; 2drop removes remainder
    drop			            \  highest possible number is 28 bit
;

\ preparing the four 8 bit numbers to send to the AD9833 via SPI

: FREQREGPREP   ( u --  freg0MSB-lsb freg0MSB-hsb freg0LSB-lsb freg0LSB-hsb )
  dup  
   %00000000001111111100000000000000 and 14 rshift
  swap dup
   %00001111110000000000000000000000 and 22 rshift %01000000 or 
  swap dup
   %00000000000000000000000011111111 and
  swap
   %00000000000000000011111100000000 and  8 rshift %01000000 or
;
  
: AD9833SPISETUP ( -- )
  2 SPI1SETUP              	        \ mode 2   
  CSSETUP-PA11
;

: AD9833-400HZ ( --  )  		\ 400 Hz test according to the application note
  CSON-PA11     		         \ CS lo
  $21 C!SPI  $00 C!SPI
  $50 C!SPI  $C7 C!SPI
  $40 C!SPI  $00 C!SPI
  $C0 C!SPI  $00 C!SPI
  $20 C!SPI  $00 C!SPI
  CSOFF-PA11                      \ CS hi
;

: AD9833WRITE ( ud -- )			\ enter frequency with decimal point at end
  FREQREGCALC
  FREQREGPREP
  CSON-PA11     		         \ CS lo
  $21 C!SPI  $00 C!SPI       		
  C!SPI C!SPI 
  C!SPI C!SPI
  $C0 C!SPI  $00 C!SPI 
  $20 C!SPI  $00 C!SPI 
  CSOFF-PA11                     \ CS hi
;


\ LTC2400

: LTC2400SETUP ( -- )
  0 SPI1SETUP                    \ mode 0
  CSSETUP-PA8                    \ using the secondary CS pin!
;

: LTC2400READ ( -- u )           \ reading the result of the latest conversion!
  CSON-PA8     		        \ CS lo
  C@SPI 
  C@SPI 
  C@SPI 
  C@SPI 
  CSOFF-PA8             \ CS hi    
\ turning the 4 bytes into a single 32 bit number with 24 significant bits
\ the least and most significant nibbles are discarded
\ the notation below is showing the 8 nibbles of the 32 bit numbers on the stack
                \	0|0|0|0|0|0|8|7		0|0|0|0|0|0|6|5		0|0|0|0|0|0|4|3		0|0|0|0|0|0|2|1
 4 rshift	\	0|0|0|0|0|0|8|7		0|0|0|0|0|0|6|5		0|0|0|0|0|0|4|3		0|0|0|0|0|0|0|2
 swap		\	0|0|0|0|0|0|8|7		0|0|0|0|0|0|6|5		0|0|0|0|0|0|0|2		0|0|0|0|0|0|4|3
 4 lshift	\	0|0|0|0|0|0|8|7		0|0|0|0|0|0|6|5		0|0|0|0|0|0|0|2		0|0|0|0|0|4|3|0
 +		\	0|0|0|0|0|0|8|7		0|0|0|0|0|0|6|5		0|0|0|0|0|4|3|2		
 swap		\	0|0|0|0|0|0|8|7		0|0|0|0|0|4|3|2		0|0|0|0|0|0|6|5
 12 lshift      \	0|0|0|0|0|0|8|7		0|0|0|0|0|4|3|2		0|0|0|6|5|0|0|0
 +              \	0|0|0|0|0|0|8|7		0|0|0|6|5|4|3|2
 swap           \	0|0|0|6|5|4|3|2          0|0|0|0|0|0|8|7
 $F AND		\       0|0|0|6|5|4|3|2          0|0|0|0|0|0|0|7
 20 lshift       \       0|0|0|6|5|4|3|2          0|0|7|0|0|0|0|0
 +               \       0|0|7|6|5|4|3|2
;

: LTC2400AVERAGEDREAD             \ ( n -- u )     	
   LTC2400READ drop               \ deleting the result from prior conversion
   DUP 		                  \ ( n -- n n )		
   0 swap 0 	                  \ ( n n -- n 0 n 0 )		
   do 200 ms LTC2400READ + loop   \ ( n 0 n 0 -- n u ) conversion time is 160 ms
   swap					
   /
;
 
