# Python 3 GUI for PAS controller
from tkinter import Tk, Label, Button, Entry, IntVar, END, W, E
import tkinter.font as font
import serial
import time
import numpy
import matplotlib
import matplotlib.pyplot as plt
matplotlib.use('TkAgg')
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
from matplotlib.figure import Figure
import matplotlib.animation as animation
from matplotlib import style

class Create_Chart:
    def __init__(self, data_x, data_y):
        self.data_x = data_x
        self.data_y = data_y
        
    def Plot(self):
        fig = plt.figure()
        ax = fig.add_subplot(111)
        line = ax.plot(self.data_x, self.data_y, marker="o", markersize=0)
        plt.xlabel('Frequency (Hz)')
        plt.ylabel('Signal (a.u.)')
        plt.title('Frequency plot')
        ymax = max(self.data_y)
        xpos = self.data_y.index(ymax)
        xmax = self.data_x[xpos]
        ymaximum = str(ymax)
        xmaximum = str(xmax)
        text = "Local maximum: (" + xmaximum + "," + ymaximum + ")"
        ax.annotate(text,
                    xy=(xmax, ymax),
                    #xytext=(xmax, ymax+5),
                    arrowprops=dict(facecolor='black',
                                    shrink=0.02),
                    )
        print(text)
       # ax.set_ylim(0,50000) # set fixed y range
        plt.show()

class PAS:
    def __init__(self, ser):
        self.ser = ser
        self.my_list_x = []
        self.my_list_y = []
        
    def build_gui(self, master):
        self.master = master
        master.title("PAS controller")
        self.master.configure(bg = 'lightblue')

        arialbig = font.Font(family='Arial', size=20)
        arialmedium = font.Font(family='Arial', size=15)
        arialsmall = font.Font(family='Arial', size=10)
        arialtiny = font.Font(family='Arial', size=8)
        arialbold = font.Font(family='Arial', size=12, weight="bold")

        self.low_limit = 0
        self.up_limit = 0
        self.fixed_frequency = 0
        self.fixed_time = 0
        self.resolution = 0
    
        self.btn_connect = Button(master, font=arialtiny, text="CONNECT", command=self.connect)
        self.btn_disconnect = Button(master, font=arialtiny, text="DISCONNECT", command=self.disconnect)
        self.btn_initialize = Button(master, font=arialtiny, text="INITIALIZE", command=self.initialize)

        self.heading = Label(master, font=arialbig, bg = 'lightblue', text="PAS CONTROLLER")

        self.heading2 = Label(master, font=arialbold, bg = 'lightblue', text="Frequency Scan")

        self.upperlimit = Label(master, font=arialsmall, bg = 'lightblue', text="Maximum Frequency (Hz)")
        self.lowerlimit = Label(master, font=arialsmall, bg = 'lightblue', text="Minimum Frequency (Hz)")

        vcmd = master.register(self.validate) # use the widgets register method for validating the entry of the textfield, returns a character string (true or false
        vcmd2 = master.register(self.validate2) # use the widgets register method for validating the entry of the textfield, returns a character string (true or false)
        vcmd3 = master.register(self.validate3) # use the widgets register method for validating the entry of the textfield, returns a character string (true or false)
        vcmd4 = master.register(self.validate4) # use the widgets register method for validating the entry of the textfield, returns a character string (true or false)
        vcmd5 = master.register(self.validate5)
        
        self.entry1 = Entry(master, font=arialsmall, bg = 'lightblue',  validate="key", validatecommand=(vcmd, '%P')) #these to methods are needed for getting the entry of a text field
        self.entry2 = Entry(master, font=arialsmall, bg = 'lightblue',  validate="key", validatecommand=(vcmd2, '%P')) #these to methods are needed for getting the entry of a text field
        self.entry3 = Entry(master, font=arialsmall, bg = 'lightblue',  validate="key", validatecommand=(vcmd3, '%P')) #these to methods are needed for getting the entry of a text field

        self.start_button = Button(master, font=arialtiny, text="START", command=self.start_scan)
        self.stop_button = Button(master, font=arialtiny, text="STOP")

        self.heading3 = Label(master, font=arialbold, bg = 'lightblue', text="Fixed Frequency")
        self.labelresolution = Label (master, font=arialsmall, bg = 'lightblue', text="Scanning Resolution")
        self.label = Label(master, font=arialsmall, bg = 'lightblue', text="Freqency (Hz)")
        self.label2 = Label(master, font=arialsmall, bg = 'lightblue', text="Time (s)")
        self.entry4 = Entry(master, font=arialsmall, bg = 'lightblue', validate="key", validatecommand=(vcmd4, '%P')) #these to methods are needed for getting the entry of a text field
        self.entry5 = Entry(master, font=arialsmall, bg = 'lightblue', validate="key", validatecommand=(vcmd5, '%P')) #these to methods are needed for getting the entry of a text field
    
        self.start_button2 = Button(master, font=arialtiny, text="START", command=self.start_fixedscan)

        self.heading4 = Label(master, font=arialbold, bg = 'lightblue', text="Chart")

        self.start_button3 = Button(master, font=arialtiny, text="START", command=self.chart)
        self.reset_button = Button(master, font=arialtiny, text="RESET", command=self.clearlist)

        # LAYOUT

        self.heading.grid(row=0, column=2, sticky=E+W, padx=10, pady=10)
        self.heading2.grid(row=1, column=2, sticky=E+W, padx=20, pady=20)
        self.lowerlimit.grid(row=2, column=1)
        self.entry1.grid(row=2, column=2)
        self.upperlimit.grid(row=3, column=1, padx=5, pady=5)
        self.entry2.grid(row=3, column=2)
        self.labelresolution.grid(row=4, column=1)
        self.entry3.grid(row=4, column=2)

        self.start_button.grid(row=2, column=3, sticky=E+W)

        self.heading3.grid(row=5, column=2, sticky=E+W, padx=20, pady=20)
        self.label.grid(row=6, column=1)
        self.entry4.grid(row=6, column=2)
        self.label2.grid(row=7, column=1)
        self.entry5.grid(row=7, column=2)
        self.start_button2.grid(row=7, column=3, sticky=E+W)

        self.heading4.grid(row=8, column=2, sticky=E+W, padx=20, pady=20)

        self.start_button3.grid(row=9, column=2, sticky=E+W)
        self.reset_button.grid(row=9, column=3, sticky=E+W)

        self.btn_connect.grid(row=10, column=2, sticky=E+W, pady=80)
        self.btn_disconnect.grid(row=10, column=3, sticky=E+W, pady=80)
        self.btn_initialize.grid(row=10, column=1, sticky=E+W, pady=80)

    def validate(self, new_text):
        if not new_text: # the field is being cleared
            self.low_limit = 0
            return True

        try:
            self.low_limit = int(new_text) # if a valid number is entered in the text field, it will be stored in self.up_limit
            return True

        except ValueError:
            return False

    def validate2(self, new_text):
        if not new_text: # the field is being cleared
            self.up_limit = 0
            return True

        try:
            self.up_limit = int(new_text) # if a valid number is entered in the text field, it will be stored in self.up_limit
            return True

        except ValueError:
            return False

    def validate3(self, new_text):
        if not new_text: # the field is being cleared
            self.resolution = 0
            return True

        try:
            self.resolution = int(new_text) # if a valid number is entered in the text field, it will be stored in self.up_limit
            return True

        except ValueError:
            return False

    def validate4(self, new_text):
        if not new_text: # the field is being cleared
            self.fixed_frequency = 0
            return True

        try:
            self.fixed_frequency = int(new_text) # if a valid number is entered in the text field, it will be stored in self.up_limit
            return True

        except ValueError:
            return False

    def validate5(self, new_text):
        if not new_text: # the field is being cleared
            self.fixed_time = 0
            return True

        try:
            self.fixed_time = int(new_text) # if a valid number is entered in the text field, it will be stored in self.up_limit
            return True

        except ValueError:
            return False
        
    def connect(self):
        print(self.ser.name)
        self.ser.reset_input_buffer()
        line = self.ser.inWaiting()
        ser.flushInput()
        ser.flushOutput()
        print("Serial connection open and I try to read ...")

    def disconnect(self):
        self.ser.close()
        time.sleep(0.4)
        print("Serial connection closed")
        root.destroy()

    def initialize(self):
        self.ser.write(b'AD9833SPISETUP\r')
        
        print("Sending: b'AD9833SPISETUP'")
        time.sleep(0.2)
       
        
        self.ser.write(b'AD9833-400HZ\r')
        
        print("Sending: b'AD9833-400HZ'")
        time.sleep(0.2)
        

        self.ser.write(b'LTC2400SETUP')
        
        print("Sending: b'LTC2400SETUP'")
        time.sleep(0.2)
       
    

    def start_scan(self):
        print ("Lower limit: " + str(self.low_limit))
        print ("Upper limit: " + str(self.up_limit))
        lower_limit = self.low_limit
        upper_limit = self.up_limit + 1
        print ("Starting to scan the frequencies...")
        self.serial_communication(lower_limit, upper_limit, self.resolution)

    def start_fixedscan(self):
        print("Fixed Frequency:" + str(self.fixed_frequency))
        print("Fixed Time:" + str(self.fixed_time))
        self.fixed_scan(self.fixed_frequency, self.fixed_time)

    def clear_data(self):
        self.my_list_x = []
        self.my_list_y = []

    def serial_communication(self, lower, upper, resolution):
        # If both my_list_x and my_list_y are not empty, clear it!
        self.clear_data()
        
        F = open('output_serial_com.txt', 'w+')
        
        for Frequency in range(lower, upper, resolution): #Set Frequency range in integer
            if not self.ser.isOpen():
                self.ser.open()
                
            Freq = str(Frequency) + '. AD9833WRITE\r' #String and integer concatenation
            Freq_encoded = Freq.encode('utf-8')
            #Freq_encoded = bytes(Freq, encoding="raw_unicode_escape")
            self.ser.write(Freq_encoded)
            Freq_decoded = Freq_encoded.decode('utf-8')
            Fscan = Freq_decoded.replace('. AD9833WRITE\r', '')
            line = self.ser.readline()
            self.ser.write(b'LTC2400READ .\r')
            line = self.ser.readline()#.decode('utf-8')
            Signal = str(line).split(' ')[2]
            time.sleep(0.13)

            # Write file
            F.writelines(str(Frequency) + ',' + Signal + '\n')
            print(f"Freq: {Frequency} and Signal: {Signal}")
            
            
            
            # Store data in the list
            self.my_list_x.append(Frequency)
            try:
                self.my_list_y.append(float(Signal))
            except:
                self.my_list_y.append(0.0)
            
            # Delay receiver
            self.ser.flush()
            self.ser.close()
            
        F.close()
        
        print("*"*10 + " Job done " + "*"*10)

    def fixed_scan(self, fixedNumber, fixedtime):
        if not self.ser.isOpen():
            self.ser.open()
        F = open('output_fixed_scan.txt', 'w+')

        for Time in range(0, fixedtime): #Set Frequency range in integer
            Freq = str(fixedNumber) + '. AD9833WRITE\r' #String and integer concatenation
            Freq_encoded = Freq.encode('utf-8')
            self.ser.write(Freq_encoded)
            Freq_decoded = Freq_encoded.decode('utf-8')
            Fscan = Freq_decoded.replace('. AD9833WRITE\r', '')
            line = self.ser.readline()
            self.ser.write(b'LTC2400READ .\r')
            line = self.ser.readline()#.decode('utf-8')
            Signal = str(line).split(' ')[2]
            time.sleep(0.98)
            F.writelines(Fscan + ',' + Signal + '\n')
            print(f"Freq: {str(Fscan)} and Signal: {Signal}")
            
        
        F.close()
        
        # Delay receiver
        self.ser.flush()
        self.ser.close()
        
        print("*"*10 + " Job done " + "*"*10)

    def chart(self):
        my_plot = Create_Chart(self.my_list_x, self.my_list_y)
        my_plot.Plot()
        
    def clearlist(self):
        self.my_list_x.clear()
        self.my_list_y.clear()
        print('XValues:', self.my_list_x)
        print('YValues:', self.my_list_y)

if __name__ == "__main__":
    ser = serial.Serial('COM6', 115200, timeout=1)
    """
        baudrate = 115200,
        bytesize = serial.EIGHTBITS, 
        parity = serial.PARITY_NONE,
        stopbits = serial.STOPBITS_ONE, 
        timeout = 1,
        xonxoff = False,
        rtscts = False,
        dsrdtr = False,
        writeTimeout = 2
        )
    """
    my_gui = PAS(ser)
    root = Tk()
    root.geometry("530x540")
    my_gui.build_gui(root)
    root.mainloop()


